"""
udp_server package
==================

This package provides an API for implementing a server that communicates with clients using the UDP Internet protocol.

Classes
-------
MyUDPHandler(DatagramRequestHandler)
    The request handler class for the UDP server.
    
Functions
---------
start_server(host: str, port: int)
    Start the UDP server and listen for incoming messages.

Usage
-----
To start the UDP server, import the `start_server()` function and call it with the host address and port number to bind to.

Example:
    
    >>> from udp_server import start_server
    >>> if __name__ == "__main__":
    >>>     start_server("localhost", 9999)
    
This will start the UDP server on `localhost` and port `9999`.
"""

import socketserver

class MyUDPHandler(socketserver.DatagramRequestHandler):
    """
    The request handler class for our server.
    """

    def handle(self):
        # self.request is the UDP socket connected to the client
        data, socket = self.request
        print("{} wrote:".format(self.client_address[0]))
        print(data)
        # send back the same message to the client
        socket.sendto(data, self.client_address)

def start_server2(host: str, port: int):
    """
    Start the UDP server and listen for incoming messages.
    
    Parameters
    ----------
    host : str
        The host address to bind to.
    port : int
        The port number to bind to.
    """
    with socketserver.UDPServer((host, port), MyUDPHandler) as server:
        print("Server running on {}:{}".format(host, port))
        try:
            server.serve_forever()
        except KeyboardInterrupt:
            pass
        finally:
            server.shutdown()
            print("Server stopped.")



#API to access various services via HTTP as a client

import urllib.request
import json

class HttpApiClient:

    """
    HttpApiClient class provides a simple API to access various services via HTTP as a client. It can perform HTTP GET, POST, PUT, PATCH, and DELETE requests to the given endpoint.

    Example Usage:
    client = HttpApiClient("https://example.com/api/v1/")
    response = client.get("users/")
    print(response)

    Attributes:
    base_url (str): Base URL for all requests.
    auth_token (str): Authorization token to be used in the header of the request.

    Methods:
    _build_request(endpoint, method, data):
    Private method that constructs a urllib.request.Request object with the given endpoint, HTTP method, and data (if provided).
    get(endpoint):
    Sends an HTTP GET request to the given endpoint and returns the response data as a JSON object.

    post(endpoint, data):
    Sends an HTTP POST request to the given endpoint with the provided data and returns the response data as a JSON object.

    put(endpoint, data):
    Sends an HTTP PUT request to the given endpoint with the provided data and returns the response data as a JSON object.

    patch(endpoint, data):
    Sends an HTTP PATCH request to the given endpoint with the provided data and returns the response data as a JSON object.

    delete(endpoint):
    Sends an HTTP DELETE request to the given endpoint and returns the response data as a JSON object.
    """

    def __init__(self, base_url, auth_token=None):
        self.base_url = base_url
        self.auth_token = auth_token

    def _build_request(self, endpoint, method="GET", data=None):
        url = self.base_url + endpoint
        headers = {}

        if self.auth_token:
            headers["Authorization"] = "Bearer " + self.auth_token

        if method in ["POST", "PUT", "PATCH"]:
            headers["Content-Type"] = "application/json"

        request = urllib.request.Request(url, headers=headers, method=method)

        if data:
            data = json.dumps(data).encode("utf-8")
            request.data = data

        return request

    def get(self, endpoint):
        request = self._build_request(endpoint, method="GET")
        response = urllib.request.urlopen(request)
        return json.loads(response.read().decode("utf-8"))

    def post(self, endpoint, data):
        request = self._build_request(endpoint, method="POST", data=data)
        response = urllib.request.urlopen(request)
        return json.loads(response.read().decode("utf-8"))

    def put(self, endpoint, data):
        request = self._build_request(endpoint, method="PUT", data=data)
        response = urllib.request.urlopen(request)
        return json.loads(response.read().decode("utf-8"))

    def patch(self, endpoint, data):
        request = self._build_request(endpoint, method="PATCH", data=data)
        response = urllib.request.urlopen(request)
        return json.loads(response.read().decode("utf-8"))

    def delete(self, endpoint):
        request = self._build_request(endpoint, method="DELETE")
        response = urllib.request.urlopen(request)
        return json.loads(response.read().decode("utf-8"))




import socket
import ssl


class SSLServer:
    """
    Provides an SSL server that can receive and respond to client requests.
    """
    
    def __init__(self, address, port, certfile, keyfile, cafile=None):
        """
        Initializes the SSL server with the specified address, port, certificate file, key file, and CA file (optional).
        :param address: The IP address of the server.
        :param port: The port number to listen on.
        :param certfile: The path to the server's SSL certificate file.
        :param keyfile: The path to the server's SSL key file.
        :param cafile: The path to the CA file used to validate client certificates (optional).
        """
        self.address = address
        self.port = port
        self.certfile = certfile
        self.keyfile = keyfile
        self.cafile = cafile
        
    def start(self):
        """
        Starts the SSL server and listens for incoming client connections.
        """
        context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
        context.load_cert_chain(certfile=self.certfile, keyfile=self.keyfile)
        if self.cafile:
            context.load_verify_locations(cafile=self.cafile)
            context.verify_mode = ssl.CERT_REQUIRED
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as server_socket:
            server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            server_socket.bind((self.address, self.port))
            server_socket.listen()
            with context.wrap_socket(server_socket, server_side=True) as ssl_socket:
                print(f"SSL server started on {self.address}:{self.port}")
                while True:
                    client_socket, client_address = ssl_socket.accept()
                    print(f"Connection from {client_address}")
                    client_socket.close()  # TODO: implement client handling logic
                    
                    
class SSLClient:
    """
    Provides an SSL client that can connect to an SSL server and send requests.
    """
    
    def __init__(self, address, port, certfile, keyfile, cafile=None):
        """
        Initializes the SSL client with the specified address, port, certificate file, key file, and CA file (optional).
        :param address: The IP address of the server.
        :param port: The port number to connect to.
        :param certfile: The path to the client's SSL certificate file.
        :param keyfile: The path to the client's SSL key file.
        :param cafile: The path to the CA file used to validate server certificates (optional).
        """
        self.address = address
        self.port = port
        self.certfile = certfile
        self.keyfile = keyfile
        self.cafile = cafile
        
    def connect(self):
        """
        Connects the SSL client to the SSL server.
        """
        context = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
        context.load_cert_chain(certfile=self.certfile, keyfile=self.keyfile)
        if self.cafile:
            context.load_verify_locations(cafile=self.cafile)
            context.verify_mode = ssl.CERT_REQUIRED
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as client_socket:
            with context.wrap_socket(client_socket, server_hostname=self.address) as ssl_socket:
                ssl_socket.sendall(b"Hello, server!")
                response = ssl_socket.recv(1024)
                print(f"Server response: {response.decode()}")




import xmlrpc.client

def execute_remote_function(url, function_name, *args):
    """
    Executes a remote function using XML-RPC.
    :param url: The URL of the remote XML-RPC server.
    :param function_name: The name of the remote function to execute.
    :param args: The arguments to pass to the remote function.
    :return: The result of the remote function.
    """
    proxy = xmlrpc.client.ServerProxy(url)
    return getattr(proxy, function_name)(*args)



#!/usr/bin/env python

import cgi
import json

class MyAPI:
    def __init__(self):
        pass

    def handle_request(self, request):
        # Get the HTTP method and request parameters
        method = request["REQUEST_METHOD"]
        params = cgi.FieldStorage()

        # Handle different methods
        if method == "GET":
            # Handle GET requests here
            response = {"message": "This is a GET request"}
        elif method == "POST":
            # Handle POST requests here
            action = params.getvalue("action")
            if action == "add":
                try:
                    num1 = int(params.getvalue("num1"))
                    num2 = int(params.getvalue("num2"))
                    result = num1 + num2
                    response = {"result": result}
                except:
                    response = {"error": "Invalid input"}
            elif action == "subtract":
                try:
                    num1 = int(params.getvalue("num1"))
                    num2 = int(params.getvalue("num2"))
                    result = num1 - num2
                    response = {"result": result}
                except:
                    response = {"error": "Invalid input"}
            else:
                response = {"error": "Unsupported action"}
        elif method == "PUT":
            # Handle PUT requests here
            response = {"message": "This is a PUT request"}
        elif method == "DELETE":
            # Handle DELETE requests here
            response = {"message": "This is a DELETE request"}
        else:
            # Handle unsupported methods
            response = {"error": "Unsupported HTTP method"}

        self.send_response(response)

    def send_response(self, response):
        # Send response as JSON
        print("Content-Type: application/json")
        print()
        print(json.dumps(response))

if __name__ == "__main__":
    # Parse the incoming request and call the handler function
    api = MyAPI("GET")
    request = cgi.FieldStorage()
    api.handle_request(request)




from setuptools import setup

setup(
    name='apitut_nirmit',
    version='0.2.3',
    description='A tutorial package for creating APIs',
    author='Nirmit Sakre',
    author_email='nirmitsakre@gmail.com',
    packages=['apitut_nirmit'],
    install_requires=[
        'requests',
        'flask',
    ],
)
